<?php
/**
 * StatusNet - the distributed open-source microblogging tool
 * Copyright (C) 2009, StatusNet, Inc.
 *
 * PHP version 5
 *
 * "Hard" StatusNet i18n -- i.e., overriding default language on subdomains
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @category  Plugin
 * @author    Julien Chaumond <chaumond@gmail.com>
 * @license   http://www.fsf.org/licensing/licenses/agpl-3.0.html AGPL 3.0
 * @link      http://status.net/
 */

if (!defined('STATUSNET')) {
    exit(1);
}



/**
 * "Hard" StatusNet i18n -- i.e., overriding default language on subdomains
 * 
 * This is most useful for Search engine crawlers, which do not set an
 * Accept-language header (so they only crawl the site's default language).
 * Most useful if you've got a lot of multilingual "static" content,
 * i.e. you use StatusNet as a CMS of sorts.
 *
 * @category  Plugin
 * @author    Julien Chaumond <julien@pnzi.com>
 */




class HardI18nPlugin extends Plugin
{
    // List of languages that get explicitly proposed through the interface.
    var $proposed_languages = array("en" => 'English', "fr" => 'Français', "es" => 'Español');
    
    // List of actions on which multilingual content is enabled (passed from config.php)
    public $on_actions = null;
    
    // Whether or not we change the "server name" : use with caution.
    public $change_server = false;
    
    
    function onPluginVersion(&$versions)
    {
        $versions[] = array('name' => 'HardI18nPlugin',
                            'version' => '0.1',
                            'author' => 'Julien Chaumond',
                            'homepage' => 'http://status.net/wiki/Plugin:HardI18nPlugin',
                            'rawdescription' =>
                            _m('"Hard" StatusNet i18n on subdomains.'));
        return true;
    }
    
    
    
    function _get_subdomain()
    {
        $urlParts = explode('.', $_SERVER['HTTP_HOST']);
        if (array_key_exists($urlParts[0], get_nice_language_list())) {
            // @see lib/language.php
            // We assume site's domain name is not a language code...
            return $urlParts[0];
        }
        else {
            return false;
        }
    }

    
    
    function onArgsInitialize($args)
    {
        // There seems to be no Hook before index.php's main() calling common_init_language()
        // So we have to do this here and call common_init_language() a second time,
        // which is less than ideal (side effects not ruled out).
        if (in_array($args['action'], $this->on_actions)) {
            if ($this->_get_subdomain()) {
                global $config;
                $config['site']['language'] = $this->_get_subdomain();
                
                if ($this->change_server) {
                    //
                    // We change the server name setting as well,
                    // to stay on the present subdomain.
                    // However this is far from perfect because e.g. links in notices are cached
                    // And we want to avoid users to create notices with subdomain-links inside them.
                    // Conclusion : Again, this is mainly for search engine crawlers, not for users.
                    //
                    // XXX: improve situation.
                    //
                    // First backup for future use.
                    $config['site']['serveroriginal'] = $config['site']['server'];
                    // Then override.
                    $config['site']['server'] = $_SERVER['HTTP_HOST'];
                }
                
                // side effects not ruled out..
                common_init_language();
            }
        }
        return true;
    }
    
    
    
    function onEndSecondaryNav($action)
    {
        // This is far from perfect and intended mostly for search engines.
        // The better way to go, from a user's perspective, would be to store 
        // desired site language in a cookie,
        // not just rely on browser's Accept-language.
        
        if (in_array($action->trimmed('action'), $this->on_actions)) {
            
            $currentLanguage = ($this->_get_subdomain()) ? $this->_get_subdomain() : "en";
            
            $l = array_diff_key($this->proposed_languages, array($currentLanguage => null));
            
            foreach ($l as $ll => $name) {
                $action->menuItem($this->curPageURL($ll), 
                                   $name);
            }
        }
    }
    
    
    
    function curPageURL($ll)
    {
        $pageURL = 'http';
        $pageURL .= "://";
        $pageURL .= $this->getSubdomainForLang($ll) . $_SERVER["REQUEST_URI"];
        return $pageURL;
    }
    
    function getSubdomainForLang($ll)
    {
        $sub = ($ll == "en") ? "" : $ll.".";
        
        $server = common_config('site', 'serveroriginal') ? common_config('site', 'serveroriginal') : common_config('site', 'server');
        
        return $sub . $server;
    }
    
}

