<?php
/**
 * Plugin to do ratings (with hReviews) inside notices
 *
 * PHP version 5
 *
 * LICENCE: This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @category  Plugin
 * @package   StatusNet
 * @author    Julien Chaumond <julien@pnzi.com>
 * @license   http://www.fsf.org/licensing/licenses/agpl-3.0.html GNU Affero General Public License version 3.0
 * @link      http://status.net/
 * @link      http://pnzi.com/
 */

if (!defined('STATUSNET')) {
    exit(1);
}

require_once INSTALLDIR . '/local/plugins/TagRating/Tag_rating.php';
require_once INSTALLDIR . '/local/plugins/TagRating/lib/ratingstars.php';

define('UTF8STAR', '★');

/**
 * Plugin to do ratings (with hReviews and hReview-aggregates) inside notices
 *
 * The way it works is, it assigns a rating (described in the notice by a string of stars)
 * to every hashtag in the notice, makes an hReview of the notice itself, and save the rating
 * itself to a table in the database.
 *
 * hReview is a semantic microformat described here: http://microformats.org/wiki/hreview
 * Google explains how they expect semantic reviews here: 
 *   http://www.google.com/support/webmasters/bin/answer.py?hl=en&answer=146645
 *
 * @category Plugin
 * @package  StatusNet
 * @author   Julien Chaumond <julien@pnzi.com>
 * @license  http://www.fsf.org/licensing/licenses/agpl-3.0.html GNU Affero General Public License version 3.0
 * @link     http://status.net/
 */

class TagRatingPlugin extends Plugin
{
    // Whether to show a "Popular Tag" section in the sidebar
    public $show_section = true;
    // Whether to show a Rating stars widget on each tag page
    public $show_on_tag_page = true;
    // If you want to set these to false, use:
    //   addPlugin('TagRating', array('show_section' => false, 'show_on_tag_page' => false));
    
    
    function onCheckSchema()
    {
        $schema = Schema::get();

        $schema->ensureTable('tag_rating',
                             array(new ColumnDef('tag', 'varchar', 
                                                 /*size*/ 64,
                                                 /*nullable*/ false,
                                                 /*key*/ 'MUL'),
                                   new ColumnDef('rating', 'int', 1),
                                   new ColumnDef('notice_id', 'int', 11)));

        return true;
    }
    
    
    function onStartNoticeSave($notice)
    {
        // We hook to this event to be able to change $content->rendered
        // before it's saved to database (we don't want to treat notices at every display).
        // If it takes too long at Web-time, we could move it to a queue.
        
        
        // First, replace the longest string of ASCII stars (e.g. "*" or "***")
        // by the equivalent string of "Unicode stars" (same number of Unicode stars).
        // This could be made optional.
        
        $match = preg_match_all("/(\*)+/", $notice->rendered, $matches);
        if ($match) {
            $stars = $this->get_longest_string($matches[0]);
            $nbstars = strlen($stars);
            $rating = str_repeat(UTF8STAR, $nbstars);
            $notice->rendered = str_replace($stars, $rating, $notice->rendered);
            // Should we change $notice->content as well?
        }
        
        
        // Then detect the longest "Unicode star" string
        // and make it a hReview "rating"
        
        $match = preg_match_all("/".UTF8STAR."+/u", $notice->rendered, $matches);
        
        // From http://www.regular-expressions.info/php.html :
        // A special option is the /u which turns on the Unicode matching mode, 
        // instead of the default 8-bit matching mode. You should specify /u for 
        // regular expressions that use \x{FFFF}, \X or \p{L} to match Unicode characters, 
        // graphemes, properties or scripts. PHP will interpret '/regex/u' 
        // as a UTF-8 string rather than as an ASCII string.
        
        if ($match) {
            $stars = $this->get_mb_longest_string($matches[0]);
            $nbstars = mb_strlen($stars);
            $rating = '<abbr class="rating" title="'.$nbstars.'">'.$stars.'</abbr>';
            $notice->rendered = str_replace($stars, $rating, $notice->rendered);
            
            // Finally, turn hashtags into hReview "items".
            // If there are several hashtags, we have no way of knowing precisely 
            // which tag the rating is about.
            // So we add the item class to each of them 
            // (another way would be to assume the rating is about this first tag).
            //
            // XXX: check that it is valid hReview syntax understandable by browsers and search engines.
            
            $notice->rendered = str_replace('<span class="tag">', '<span class="tag item">', $notice->rendered);
            
            // If the notice doesn't contain any hashtag,
            // we could try to find a URL and rate it, but for now we just do nothing.
        }
        
        return true;
    }
    
    
    function onEndNoticeSave($notice)
    {
        // The rating itself (i.e. saving to database)
        // happens here because we now have a notice_id (we didn't before).
        
        if ($this->containsRating($notice->rendered)) {
            
            // First, we need to find back the rating from $notice->rendered
            
            // Fragment of HTML containing the rating in $notice->rendered
            $ratingHtmlFragment = '<abbr class="rating" title="';
            // Position of rating in the notice string
            $ratingPosition = strpos($notice->rendered, $ratingHtmlFragment) + strlen($ratingHtmlFragment);
            // Extract rating (assume it's a single digit)
            $nbstars = substr($notice->rendered, $ratingPosition, 1);
            $nbstars = (int) $nbstars;
            
            // We use the tag regexp definition from classes/Notice.php.

            $count = preg_match_all('/(?:^|\s)#([\pL\pN_\-\.]{1,64})/', $notice->content, $m);
            
            if (!$count) {
                // This notice contains no hashtag.
                return true;
            }
            
            foreach ($m[1] as $tag) {
                Tag_rating::saveRating($tag, $nbstars, $notice->id);
            }
            // This is an important design decision in this plugin:
            // We store every tag again even though right now we assign the same rating
            // to every tag in the notice, because it might now be always the case.
            // (We might want to filter which tag we assign the rating to later, for example).
        }
        
        return true;
    }
    
    
    function onStartShowNoticeItem($action)
    {
        // Make a hReview only of the notices that have a rating attached.
        
        if ($this->containsRating($action->notice->rendered)) {
            $action->out->elementStart('div', array('class' => 'hreview'));
        }
        return true;
    }
    
    
    function onEndShowNoticeItem($action)
    {
        // Same here
        
        if ($this->containsRating($action->notice->rendered)) {
            $action->out->elementEnd('div');
        }
        return true;
    }
    
    
    /**
     * Tests whether a notice contains a rating or not
     *
     * @return boolean
     */
     
    function containsRating($notice_rendered) {
        return strpos($notice_rendered, '<abbr class="rating"');
    }
    
    
    /**
     * Ensure that tag_ratings for a notice are deleted
     * when that notice gets deleted.
     *
     * @param User $user
     * @param array &$related list of related tables; entries
     *              with matching user_id will be deleted.
     *
     * @return boolean hook result
     */

    function onNoticeDeleteRelated($notice, &$related)
    {
        $related[] = 'tag_rating';
        return true;
    }
    
    
    
    
    function get_mb_longest_string($array)
    {
        $temp = 0;
        $sttmp = "";
        foreach ($array as $index => $st) {
            if (mb_strlen($st) > $temp) {
                $temp = mb_strlen($st);
                $sttmp = $st;
            }
        }
        return $sttmp;
    }
    
    
    
    function onStartTagShowContent($action)
    {
        if ($this->show_on_tag_page) {
            // The following two elements are hidden, and get posted before and after
            // the rating itself in the notice text.
            
            $action->element('span', array('id' => 'SNRateA'), 
                                     sprintf(_m('#%1$s'),
                                             $action->trimmed('tag')));
            $action->element('span', array('id' => 'SNRateB'), 
                                     common_local_url('tag', array('tag' => $action->trimmed('tag'))));
            
            $rs = new RatingStars($action, $action->trimmed('tag'));
            $rs->show();
        }
        
        return true;
    }
    
    
    function onEndShowScripts($action)
    {
        $action->script(common_path('local/plugins/TagRating/js/ratingstars.js'));
    }
    
    
    function onEndShowSections($action)
    {
        if ($this->show_section) {
            $action->elementStart('div', array('class' => 'section'));
            $action->element('h2', null, _m("Popular tags"));
            $action->elementStart('ul');
            
            $tag = Tag_rating::getMostPopular();
            
            while ($tag->fetch()) {
                $action->elementStart('li');
                $action->element('a', array('href' => common_local_url('tag', array('tag' => $tag->tag))),
                                      "#".$tag->tag);
                $action->text(' (' . round($tag->rt, 2) . ')');
                $action->elementEnd('li');
            }
            
            $action->elementEnd('ul');
            $action->elementEnd('div');
        }
        
        return true;
    }
    
    
    function onEndShowStyles($action)
    {
        // Custom styles to make the rating stars widget look good
        
        $action->style('#ratingstars { float: right; } '.
                       '#ratingstars img { display: inline; cursor: pointer; } '.
                       '#ratingstars span.rating { font-weight: bolder; } '.
                       'span.votecount { color:grey; } '.
                       '#SNRateA, #SNRateB { display: none; }');
        return true;
    }
    
    
    function onPluginVersion(&$versions)
    {
        $versions[] = array('name' => 'TagRating',
                            'version' => '0.1',
                            'author' => 'Julien Chaumond',
                            'homepage' => 'http://status.net/wiki/Plugin:TagRating',
                            'rawdescription' =>
                            _m('Plugin to do ratings (with hReviews and hReview-aggregates) inside notices, based on hashtags.'));
        return true;
    }
}

