<?php
/**
 * Data class for tag ratings
 *
 * PHP version 5
 *
 * @category Data
 * @package  StatusNet
 * @author   Julien Chaumond <julien@pnzi.com>
 * @license  http://www.fsf.org/licensing/licenses/agpl.html AGPLv3
 * @link     http://status.net/
 *
 * StatusNet - the distributed open-source microblogging tool
 * Copyright (C) 2009, StatusNet, Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.     See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

if (!defined('STATUSNET')) {
    exit(1);
}

require_once INSTALLDIR . '/classes/Memcached_DataObject.php';



class Tag_rating extends Memcached_DataObject
{
    public $__table = 'tag_rating';         // table name
    public $tag;                            // varchar(64)  not_null
    public $rating;                         // int(1)
    public $notice_id;                      // int(11)

    /**
     * Get an instance by key
     *
     * This is a utility method to get a single instance with a given key value.
     *
     * @param string $k Key to use to lookup
     * @param mixed  $v Value to lookup
     *
     * @return  object found, or null for no hits
     *
     */

    function staticGet($k, $v=null)
    {
        return Memcached_DataObject::staticGet('Tag_rating', $k, $v);
    }

    /**
     * return table definition for DB_DataObject
     *
     * DB_DataObject needs to know something about the table to manipulate
     * instances. This method provides all the DB_DataObject needs to know.
     *
     * @return array array of column definitions
     */

    function table()
    {
        return array('tag' => DB_DATAOBJECT_STR + DB_DATAOBJECT_NOTNULL,
                     'rating' => DB_DATAOBJECT_INT,
                     'notice_id' => DB_DATAOBJECT_INT);
    }

    /**
     * return key definitions for DB_DataObject
     *
     * DB_DataObject needs to know about keys that the table has; this function
     * defines them.
     *
     * @return array key definitions
     */
    
    // XXX: is it really a 'K' (it's just an index)?

    function keys()
    {
        return array('tag' => 'K');
    }

    /**
     * return key definitions for Memcached_DataObject
     *
     * Our caching system uses the same key definitions, but uses a different
     * method to get them.
     *
     * @return array key definitions
     */

    function keyTypes()
    {
        return $this->keys();
    }


    /**
     * Save a new tag rating
     *
     * @return boolean
     */
    
    static function saveRating($tag, $nbstars, $notice_id)
    {
        // To have $notice->id populated,
        // we need to call saveRating() from onEndNoticeSave, not onStartNoticeSave
        
        $tr = new Tag_rating();

        $tr->tag        = $tag;
        $tr->rating     = $nbstars;
        $tr->notice_id  = $notice_id;

        $result = $tr->insert();

        if (!$result) {
            throw Exception(sprintf(_m("Could not save tag rating for %s"),
                                    $tag));
        }
        
        return true;
    }
    
    
    static function getRating($tag)
    {
        $ratings = new Tag_rating();
        
        $ratings->tag = $tag;
        
        $ratings->selectAdd();
        $ratings->selectAdd('COUNT(*) AS cnt');
        $ratings->selectAdd('AVG(rating) AS rt');
        
        $ratings->groupBy('tag');

        $cnt = $ratings->find();

        if ($cnt > 0) {
            $ratings->fetch();
            return array('count' => $ratings->cnt, 'avg' => $ratings->rt);
        }
        else {
            return array('count' => 0, 'avg' => 0);
        }
    }
    
    
    
    static function getMostPopular()
    {
        // Example of a "Most Popular" function. Adapt to application.
        
        $ratings = new Tag_rating();
        
        $ratings->selectAdd();
        $ratings->selectAdd('tag, AVG(rating) AS rt');
        $ratings->groupBy('tag');
        $ratings->orderBy('AVG(rating) DESC');
        $ratings->limit(0, 6);
        $ratings->find();
        
        return $ratings;
    }
    
    
    
    static function getMostPopularProducts()
    {
        // @See getMostPopular()
        
        $ratings = new Tag_rating();
        
        $ratings->query(sprintf('SELECT tag, AVG(rating) AS rt, urlname, hbrand '.
                                'FROM %s.products_unique AS u '.
                                'INNER JOIN tag_rating AS t ON t.tag = u.hident '.
                                'GROUP BY tag '.
                                'ORDER BY AVG(rating) DESC '.
                                'LIMIT 0, 6', 
                                common_config('db', 'products')));
        // Check query performance. Check inner join order: maybe it should be the other way around.
        
        
//        // Tentative to join through DataObject: impossible.
//        
//        $ratings = new Tag_rating();
//        $ratings->selectAdd();
//        $ratings->selectAdd('tag, AVG(rating) AS rt, urlname, hbrand');
//        $products = new Product();
//        $products->whereAdd('tag_rating.tag = products_unique.hident');
//        $ratings->joinAdd($products, array('joinType' => "INNER", 
//                                           'joinAs' => "products_unique", 
//                                           'joinCol' => "hident", 
//                                           'useWhereAsOn' => true));
//        $ratings->groupBy('tag');
//        $ratings->orderBy('AVG(rating) DESC');
//        $ratings->limit(0, 6);
//        $ratings->find();
//        
//        // Outputs something like:
//        SELECT  tag, AVG(rating) AS rt
//         FROM tag_rating
//         LEFT JOIN pnzi.products_unique AS products_unique ON (products_unique.hident=tag_rating.hident)
//         GROUP BY tag 
//         ORDER BY AVG(rating) DESC 
//         LIMIT 0, 6
        
        return $ratings;
    }
    
    
    static function getMostPopularMerchants()
    {
        // @See getMostPopular()
        
        $ratings = new Tag_rating();
        
        $ratings->query(sprintf('SELECT tag, AVG(rating) AS rt '.
                                'FROM tag_rating AS t '.
                                'INNER JOIN %s.downloads_source AS s ON t.tag LIKE s.hname '.
                                'GROUP BY tag '.
                                'ORDER BY AVG(rating) DESC '.
                                'LIMIT 0, 6',
                                common_config('db', 'products')));
        // Check query performance. Check inner join order: maybe it should be the other way around.
        // XXX: "LIKE BINARY": to make the string comparison case-sensitive.
        
        return $ratings;
    }
    
}
