<?php
/**
 * StatusNet, the distributed open-source microblogging tool
 *
 * Plugin to add Twitter and Facebook Login Buttons in the primary nav, and "disabling" registration menu item
 *
 * PHP version 5
 *
 * LICENCE: This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @category  Plugin
 * @package   StatusNet
 * @author    Julien Chaumond <chaumond@gmail.com>
 * @copyright 2009 StatusNet, Inc.
 * @license   http://www.fsf.org/licensing/licenses/agpl-3.0.html GNU Affero General Public License version 3.0
 * @link      http://status.net/
 */

if (!defined('STATUSNET')) {
    exit(1);
}


if (!common_config('twitter', 'consumer_key') || !common_config('facebook', 'apikey')) {
    echo _m("Needs both TwitterBridge and Facebook plugins activated");
    exit(1);
}


/**
 * Plugin to add Twitter and Facebook Login Buttons in the primary nav, and "disabling" registration menu item
 *
 * Needs both TwitterBridge and Facebook plugins activated
 *
 * If buttons is set to true, you get real 'Connect' buttons. 
 * Otherwise you get small icons linking to the 'Connect' pages on your StatusNet site.
 * Facebook button requires a hook to be added to FacebookPlugin.php (see below)
 *
 *     addPlugin('TwitterAndFacebookLogin', array('buttons' => true));
 *
 * @category Plugin
 * @package  StatusNet
 * @author   Julien Chaumond <chaumond@gmail.com>
 * @license  http://www.fsf.org/licensing/licenses/agpl-3.0.html GNU Affero General Public License version 3.0
 * @link     http://status.net/
 */

class TwitterAndFacebookLoginPlugin extends Plugin
{
    public $buttons = false;
    
    
    /**
     * Overides totally the register action 
     * We have to do that because there are a few other "register" links around (not just in the Primary Nav)
     */
    function onArgsInitialize($args)
    {
        if ($args['action'] == 'register') {
            $args['action'] = 'twitterlogin';
            //$args['p'] = 'main/twitterlogin';
            //Is it necessary?
        }
    }
    
    
    function onEndShowStyles($action)
    {
        // A specific CSS file just for that one line would be a bit overkill
        $action->style('#site_nav_global_primary img { vertical-align:middle; display:inline; }');
        if ($this->buttons) {
            // Buttons otherwise look cramped
            $action->style('#site_nav_global_primary { margin-bottom:30px; }');
        }
        return true;
    }
    
    
    /**
     * Force Facebook scripts and styles on every page if 'buttons' is set to it
     * To be able to do that, the following hook is added to the 
     * beginning of FacebookPlugin.php's reqFbScripts($action):
     *          if (Event::handle('ForceReqFbScripts', array($action))) {
     *              return true;
     *          }
     */
    function onForceReqFbScripts($action)
    {
        // Force Facebook scripts and styles on every page if 'buttons' is set to it
        // This hook is added to FacebookPlugin.php
        if ($this->buttons) {
            return true;
        }
        return false;
    }
    


    function onStartPrimaryNav($action)
    {
        if (!common_logged_in()) {
            
            $action->elementStart('li');
            if ($this->buttons) {
                // We should find a way to not rewrite these three lines from Facebook/FBConnectLogin.php
                // Is there some PHP construct I'm missing?
//                $action->element('fb:login-button', 
//                                 array('onlogin' => 'goto_login()',
//                                       'length' => 'long'
//                                       // 'perms' => 'email'
//                                       ));
                $action->elementStart('a', array('href' => common_local_url('twitterauthorization',
                                                                          null,
                                                                          array('signin' => true))));
                $action->element('img', array('src' => common_path('local/kGCxkZx-uZa.gif'),
                                            'alt' => 'Sign in with Twitter'));
                $action->elementEnd('a');
            }
            else {
                $action->elementStart('a', array('href' => common_local_url('FBConnectLogin'), 
                                            'title' => 'Login with your Facebook Account'));
                $action->element('img', array('src' => common_path('local/plugins/TwitterAndFacebookLogin/fbfavicon.ico'),
                                            'alt' => 'Login with your Facebook Account'));
                $action->elementEnd('a');
            }
            $action->elementEnd('li');
            
            
            $action->elementStart('li');
            if ($this->buttons) {
                // We should find a way to not rewrite these three lines from TwitterBridge/twitterlogin.php
                // Is there some PHP construct I'm missing?
                $action->elementStart('a', array('href' => common_local_url('twitterauthorization',
                                                                          null,
                                                                          array('signin' => true))));
                $action->element('img', array('src' => common_path('plugins/TwitterBridge/Sign-in-with-Twitter-lighter.png'),
                                            'alt' => 'Sign in with Twitter'));
                $action->elementEnd('a');
            }
            else {
                $action->elementStart('a', array('href' => common_local_url('twitterlogin'), 
                                            'title' => 'Login with your Twitter Account'));
                $action->element('img', array('src' => common_path('local/plugins/TwitterAndFacebookLogin/twitter.ico'),
                                            'alt' => 'Login with your Twitter Account'));
                $action->elementEnd('a');
            }
            $action->elementEnd('li');
            
        }
        return true;
    }
    
    
    /**
     * Suppress the register local menu item
     * This means registration is still open, but you'll have to type the URL yourself
     * [This is not true if the first function (onArgsInitialize) is activated]
     *
     * @return void
     */
    function onStartLoginGroupNav($action)
    {
        $action_name = $action->trimmed('action');
        $action->menuItem(common_local_url('login'),
                                _('Login'),
                                _('Login with a username and password'),
                                $action_name === 'login');

        Event::handle('EndLoginGroupNav', array(&$action));
        return false;
    }
    
    
//    function onEndShowScripts($action)
//    {
//        // This is now the preferred way (async) of loading FB's javascript,
//        // cf. http://developers.facebook.com/docs/reference/javascript/
//        
//        $action->element('div', array('id' => 'fb-root'));
//
//        $js  = '  window.fbAsyncInit = function() {';
//        $js .= '    FB.init({appId: "%s", status: true, cookie: true,';
//        $js .= '             xfbml: true});';
//        $js .= '  };';
//        $js .= '  (function() {';
//        $js .= '    var e = document.createElement("script"); e.async = true;';
//        $js .= '    e.src = document.location.protocol +';
//        $js .= '      "//connect.facebook.net/en_US/all.js";';
//        $js .= '    document.getElementById("fb-root").appendChild(e);';
//        $js .= '  }());';
//        
//        $appid = common_config('facebook', 'appid');
//        
//        $js = sprintf($js, $appid);
//        
//        // Compress the bugger down a bit
//
//        $js = str_replace('  ', '', $js);
//        
//        $action->inlineScript($js);
//
//        // XXX: no i18n for now (en_US can be replaced by fr_FR, etc.), because:
//        // 1. StatusNet's common_language() outputs two-letter language code
//        // 2. the like button, for instance, actually looks weird in some language other than en.
//    }
    
    
    
    // @see  function onEndShowSections($action) in TwitterAndFacebookAdvancedPlugin.php
    // which is over there for calling order in Aside.
}

